#ifndef PETSPECTFUSIONVIEWER_H
#define PETSPECTFUSIONVIEWER_H

#include <QMainWindow>
#include <string>

#include "RegObserver.h"
#include "RegistrationView.h"
#include "typedefs.h"

class QRegistrationThread;
class RegistrationViewDialog;

 /**
 *  
 *  $Id: PetSpectFusionViewer.h 41:8e13ae3bc48b 2009-11-20 15:35 +0100 jensen $
 *  $Lastlog: Cleanup work, fixed an issue with windows cppunit fixes #20 $
 * 
 * \brief This is the viewer app for the PetSpectFusion main programm. This class contains the main application window
 *			and uses the RegistrationView class for displaying slice views of the images.
 *
 * \authors Brian Jensen
 *          <br>
 *          {jensen}\@cs.tum.edu
 * \ingroup PetSpectFusion
 * \version 1.0
 * \date 16.12.2007
 *
 * \par License:
 * Copyright (c) 2007 - 2009,
 * This programm was created as part of a student research project in cooperation
 * with the Department for Computer Science, Chair XVI
 * and the Nuklearmedizinische Klinik, Klinikum Rechts der Isar
 *
 * <br>
 * <br>
 * All rights reserved.
 * <br>
 * <br>
 * See <a href="COPYRIGHT.txt">COPYRIGHT.txt</a> for details.
 * <br>
 * <br>
 * This software is distributed WITHOUT ANY WARRANTY; without even 
 * <br>
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * <br>
 * PURPOSE.  See the <a href="COPYRIGHT.txt">COPYRIGHT.txt</a> notice
 * for more information.
 *
 */
class QPetSpectFusionViewer : public QMainWindow
{
	Q_OBJECT
	
	public:
		QPetSpectFusionViewer(std::string fFileName, std::string mFileName, std::string oFileName, int bins, 
				float sRate, float minStep, float maxStep, int maxIter, OptimizerScalesType scales, int zoomF);

		void setMultiRes(int levs);
		void setParameters(ParametersType curParams);
		ParametersType getParameters();
		double getMetricVal();
		int getIteration();
		void setPlotFile(std::string fileName);
		void doAutomaticReg();
		void openFixedFileName(std::string fileName);
		void openMovingFileName(std::string fileName);	
	
		std::string& getFixedFileName();
		std::string& getMovingFileName();
		std::string& getOutputFileName();
		bool isRegistrationRunning();
		QRegistrationView* getRegistrationView();
	
	public slots:
		void regViewDialogClosed();
		void doReg();

	
	 protected:
		void closeEvent(QCloseEvent *event);

	private slots:
		void registrationUpdate(ParametersType curParams, double metricVal, int iteration);
		void openFixedImage();
		void openMovingImage();
		void saveAll();
		void saveRegisteredImg();
		void saveCheckDiffImg();
		void saveAbsDiffImg();
		void saveTransformParams();
		void evalCurrent();
		void about();
		void setTranslationParams(double val);
		void setVersorParams(double val);
		void setEulerParams(double val);
		void registrationDone();
		
		void setZoom1();
		void setZoom2();
		void setZoom3();
		void setZoom4();
		void setZoom6();
		void setInterp1();
		void setInterp2();
		void showCheckered();
		void showSettingsDialog();
		void showViewOptionsDialog();
	
	private:
		void createActions();
		void createComponents();
		void createMenus();
	
		void enableSettings(bool enable);
		
		QMenu *fileMenu;
		QMenu *saveMenu;
		QMenu *viewMenu;
		QMenu *zoomMenu;
		QMenu *interpolationMenu;
		QMenu *regMenu;
		QMenu *helpMenu;
		 
		QRegistrationView *regView;
		
		QAction *openFixedImageAct;
		QAction *openMovingImageAct;
		QAction *saveRegisteredAct;
		QAction *saveAbsDiffAct;
		QAction *saveCheckDiffAct;
		QAction *saveTransformParamsAct;
		QAction *saveAllAct;
		QAction *exitAct;
		QAction *doRegAct;
		QAction *aboutAct;
		QAction *checkeredAct;
		QAction *zoom1Act;
		QAction *zoom2Act;
		QAction *zoom3Act;
		QAction *zoom4Act;
		QAction *zoom6Act;
		QAction *interp1Act;
		QAction *interp2Act;
		QAction *multiResAct;
		QAction *regSettingsAct;
		QAction *viewOptionsAct;
	
		QDoubleSpinBox *xTransBox;
		QDoubleSpinBox *yTransBox;
		QDoubleSpinBox *zTransBox;
		QDoubleSpinBox *xVerRotBox;
		QDoubleSpinBox *yVerRotBox;
		QDoubleSpinBox *zVerRotBox;
		QDoubleSpinBox *xEulRotBox;
		QDoubleSpinBox *yEulRotBox;
		QDoubleSpinBox *zEulRotBox;
	
		QLabel *metricLabel;
		QLabel *iterationLabel;
		
		QPushButton *registerButton;
		QPushButton *evaluateButton;
		
		QRegistrationThread* regThread;
		
		std::string fixedFileName;
		std::string movingFileName;
		std::string outputFileName;
		
		InternalImageType::Pointer fixedImage;
		InternalImageType::Pointer movingImage; 
		ParametersType transformParams;
		OptimizerScalesType optimizerScales;

		//registration parameters
		std::string plotFileName;
		bool regIsRunning;
		bool showCheckeredImg;
		bool automatic_mode;
		bool plotting;
		int bins;
		int levels;
		int maxIterations;
		float sampleRate;
		float minStepSize;
		float maxStepSize;
	
		//registrationView dialog types
		RegistrationViewDialog* regViewDialog;
		bool regViewDialogOpen;
		bool updateAngles;
		
};

#endif