//
//  $Id: ImageIO.cxx 39:803510a3c9e1 2009-11-18 20:37 +0100 jensen $
//  $Lastlog: large refactoring for consistent initialization, unit test added, refs #2 #17 #18 $
//
//  ImageIO.cxx
//  PetSpectFusion
//
//	Created 16/12/2007, Implements various functions for reading and writing images
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include "PetSpectFusionApp.h"
#include "typedefs.h"
#include "Registration.h"
#include "PetSpectFusionViewer.h"

#include <QtGui>
 
/*
*	function does an absoltues difference from the original and registered images and returns the result image
*/
OutputImageType::Pointer getAbsoluteDifference(InternalImageType::Pointer original, InternalImageType::Pointer registered)
{
	typedef itk::SubtractImageFilter<InternalImageType, InternalImageType, InternalImageType> DifferenceFilterType;
	DifferenceFilterType::Pointer difference = DifferenceFilterType::New();
	
	difference->SetInput1(original);
	difference->SetInput2(registered);
	
	typedef itk::RescaleIntensityImageFilter<InternalImageType, OutputImageType> RescalerType;
	RescalerType::Pointer intensityRescaler = RescalerType::New();
	
	intensityRescaler->SetInput(difference->GetOutput());
	intensityRescaler->SetOutputMinimum(0);
	intensityRescaler->SetOutputMaximum(255);
	intensityRescaler->Update();
	
	return intensityRescaler->GetOutput();

}

/*
*	function does an checkerboard composite from the original and registered images and returns the result image
*/
OutputImageType::Pointer getCheckerBoardImage(InternalImageType::Pointer fixedImage, InternalImageType::Pointer movingImage)
{
	typedef itk::CheckerBoardImageFilter<OutputImageType> CheckerBoardFilterType;
	CheckerBoardFilterType::Pointer checker = CheckerBoardFilterType::New();
	
	typedef itk::RescaleIntensityImageFilter<InternalImageType, OutputImageType> RescalerType;
	RescalerType::Pointer fixedRescaler = RescalerType::New();
	RescalerType::Pointer movingRescaler = RescalerType::New();
	
	//rescale both images, because intensity differences between modalities could drown out one image
	fixedRescaler->SetInput(fixedImage);
	fixedRescaler->SetOutputMinimum(0);
	fixedRescaler->SetOutputMaximum(255);
	
	movingRescaler->SetInput(movingImage);
	movingRescaler->SetOutputMinimum(0);
	movingRescaler->SetOutputMaximum(255);
	
	checker->SetInput1(fixedRescaler->GetOutput());
	checker->SetInput2(movingRescaler->GetOutput());
	checker->Update();
	
	return checker->GetOutput();
}


/*
*	function reads in an mhd file and returns a point to an image of InternalImageType
*/
InternalImageType::Pointer loadMhdImage(const char* fileName)
{
	typedef itk::ImageFileReader< InternalImageType > MhdImageReaderType;
	MhdImageReaderType::Pointer mhdReader = MhdImageReaderType::New();
	
	mhdReader->SetFileName(fileName);
	try
	{
		mhdReader->Update();
	}
	catch(itk::ExceptionObject & err)
	{
		std::cerr << "Error reading the image from the disk!" << std::endl; 
		std::cerr << err << std::endl;
		
		if(PetSpectFusionApp::graphics_mode && PetSpectFusionApp::mainViewer)
			QMessageBox::critical(PetSpectFusionApp::mainViewer, QString("Error loading image"), 
				QString("There was an error loading image data see the console / log for more details."));
		
		
		return NULL;
	}
	
	return mhdReader->GetOutput();
}

/*
*	funciton writes an image to disk, using the OutputImageType type and filename (without ending)
*/
void writeOutputImageMhd(const char* fileName, OutputImageType::Pointer image)
{

	typedef itk::ImageFileWriter< OutputImageType> FileWriterType;
	FileWriterType::Pointer fileWriter = FileWriterType::New();

	typedef itk::MetaImageIO ImageIOType;
	ImageIOType::Pointer metaIO = ImageIOType::New();
	metaIO->SetByteOrderToLittleEndian();

	fileWriter->SetImageIO(metaIO);
	fileWriter->SetFileName(fileName);
	fileWriter->SetInput(image);
	
	try 
	{ 
		fileWriter->Update();
	} 
	catch( itk::ExceptionObject & err ) 
	{ 
		std::cerr << "Error writing the image to disk!" << std::endl; 
		std::cerr << err << std::endl;
		
		if(PetSpectFusionApp::graphics_mode && PetSpectFusionApp::mainViewer)
			QMessageBox::critical(PetSpectFusionApp::mainViewer, QString("Error writing image"), 
				QString("There was an error writing the image data see the console / log for more details."));
	} 

}

/**
 *	Function saves the transform parameters after the registration
 *
 */
void writeTransformFile(std::string fileName, TransformType::Pointer transform)
{
	int pos = fileName.find_last_of(".");
	std::string transformName = fileName.substr(0, pos).append("_transform.txt");
	
	std::ofstream transformFile;
	transformFile.open(transformName.c_str());
	if(transformFile.is_open())
	{
		ParametersType params = transform->GetParameters();
		//Every parameter on its own line, make for easy input parsing
		transformFile << params[0] << std::endl << params[1] << std::endl << params[2] << std::endl << params[3] << std::endl << params[4] << std::endl << params[5] << std::endl;
		transformFile.close();
	}
	else
		std::cerr << "Error writing the transform file: " << transformName << std::endl;
}

/**
 *	Function is used to save the registrered moving image in the fixed images space
 *	Also calls writeTransformFile in order to save the transform params
 *
 */
void writeRegisteredImage(const char* fileName, InternalImageType::Pointer fixedImage, InternalImageType::Pointer movingImage,
							ParametersType transformParams)
{
	typedef itk::ResampleImageFilter<InternalImageType, InternalImageType> ResampleFilterType;
	ResampleFilterType::Pointer resampler = ResampleFilterType::New();

	TransformType::Pointer transform = TransformType::New();
	initializeTransform(transform, fixedImage, movingImage);
	transform->SetParameters(transformParams);

	resampler->SetTransform(transform);
	resampler->SetInput(movingImage);
	resampler->SetSize(fixedImage->GetLargestPossibleRegion().GetSize());
	resampler->SetOutputOrigin( fixedImage->GetOrigin());
	resampler->SetOutputSpacing( fixedImage->GetSpacing());
	resampler->SetDefaultPixelValue(0);
	resampler->Update();
	
	typedef itk::CastImageFilter<InternalImageType, OutputImageType> CastFilterType;
	CastFilterType::Pointer caster = CastFilterType::New();
	caster->SetInput(resampler->GetOutput());

	writeOutputImageMhd(fileName, caster->GetOutput());
	std::string fName = fileName;
	writeTransformFile(fName, transform);
}

/**
 *	Function reads a saved transform file
 *
 */
void readTransformFile(std::string fileName, TransformType::Pointer transform)
{
	std::ifstream transformFile;
	transformFile.open(fileName.c_str());
	if(transformFile.is_open())
	{
		ParametersType params(6);
		double d;
		for(int i = 0; i < 6; i++)
		{
				transformFile >> d;
				params.SetElement(i, d); 
		}
		transform->SetParameters(params);
	
	}
	else
		std::cerr << "Error opening specified transform file: " << fileName << std::endl;

}

/*
*	Broaken!! FIXME
*	This function should write a dicom image series, but doesn't. I haven't fixed it yet, because I don't need dicom images
*/
void writeDicomImage(const char* dirName, OutputImageType::Pointer image)
{
	
	typedef itk::GDCMImageIO	ImageIOType;
	ImageIOType::Pointer gdcmIO =	ImageIOType::New();
	typedef itk::NumericSeriesFileNames NamesGeneratorType;
	
	itksys::SystemTools::MakeDirectory(dirName);
	
	const unsigned int      OutputDimension = 2;
	typedef itk::Image< OutputPixelType, OutputDimension >    Image2DType;
	typedef itk::ImageSeriesWriter<OutputImageType, Image2DType >  SeriesWriterType;
	NamesGeneratorType::Pointer namesGenerator = NamesGeneratorType::New();

	itk::MetaDataDictionary & dict = gdcmIO->GetMetaDataDictionary();
	std::string tagkey, value;
	tagkey = "0008|0060"; // Modality
	value = "PET";
	itk::EncapsulateMetaData<std::string>(dict, tagkey, value );
	tagkey = "0008|0008"; // Image Type
	value = "DERIVED\\SECONDARY";
	itk::EncapsulateMetaData<std::string>(dict, tagkey, value);
	tagkey = "0008|0064"; // Conversion Type
	value = "DV";
	itk::EncapsulateMetaData<std::string>(dict, tagkey, value);

	SeriesWriterType::Pointer seriesWriter = SeriesWriterType::New();

	seriesWriter->SetInput(image);
	seriesWriter->SetImageIO( gdcmIO );

	OutputImageType::RegionType region = 
		image->GetLargestPossibleRegion();

	OutputImageType::IndexType start = region.GetIndex();
	OutputImageType::SizeType  size  = region.GetSize();

	std::string format = dirName;
  
	format += "/image%03d.dcm";

	namesGenerator->SetSeriesFormat( format.c_str() );

	namesGenerator->SetStartIndex( start[2] );
	namesGenerator->SetEndIndex( start[2] + size[2] - 1 );
	namesGenerator->SetIncrementIndex( 1 );

	seriesWriter->SetFileNames( namesGenerator->GetFileNames() );
  
	try
    {
		seriesWriter->Update();
    }
	catch( itk::ExceptionObject & excp )
    {
		std::cerr << "Exception thrown while writing the dicom series " << std::endl;
		std::cerr << excp << std::endl;
    }	
}
