//
//  $Id: PetSpectFusionApp.cxx 39:803510a3c9e1 2009-11-18 20:37 +0100 jensen $
//  $Lastlog: large refactoring for consistent initialization, unit test added, refs #2 #17 #18 $
//
//  PetSpectFusionApp.cxx
//  PetSpectFusion
//
//	Created 17/11/2009, Implementing the application entry point and is called directly by main
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include "ImageIO.h"
#include "Log.h"
#include "PetSpectFusionApp.h"

using namespace PetSpectFusionApp;

float PetSpectFusionApp::sampleRate;
float PetSpectFusionApp::minStepSize;
float PetSpectFusionApp::maxStepSize;

int PetSpectFusionApp::bins;
int PetSpectFusionApp::maxIterations;
int PetSpectFusionApp::levels;
int PetSpectFusionApp::zoomFactor;

std::string PetSpectFusionApp::fixedFileName;
std::string PetSpectFusionApp::movingFileName;
std::string PetSpectFusionApp::outputFileName;
std::string PetSpectFusionApp::plotFileName;
std::string PetSpectFusionApp::logFileName;

bool PetSpectFusionApp::doMultiRes;
bool PetSpectFusionApp::automatic_mode;
bool PetSpectFusionApp::transform_mode;
bool PetSpectFusionApp::graphics_mode;

ParametersType PetSpectFusionApp::transformParams;
OptimizerScalesType PetSpectFusionApp::optimizerScales;
TransformType::Pointer PetSpectFusionApp::transform;

QPetSpectFusionViewer* PetSpectFusionApp::mainViewer;

void PetSpectFusionApp::init()
{
	sampleRate = DEFAULT_SAMPLERATE;
	minStepSize = DEFAULT_MINSTEP;
	maxStepSize = DEFAULT_MAXSTEP;
	
	bins = DEFAULT_BINS;
	maxIterations = DEFAULT_MAXITER;
	levels = DEFAULT_MULTIRES_LEVELS;
	zoomFactor = DEFAULT_ZOOM_FACTOR;
	
	fixedFileName = "";
	movingFileName = "";
	outputFileName = "";
	plotFileName = "";
	logFileName = "";
	
	doMultiRes = DEFAULT_DO_MULTIRES;
	automatic_mode = false;
	transform_mode = false;
	graphics_mode = false;
	
	transformParams = ParametersType(6);
	optimizerScales = OptimizerScalesType(6);
	
	optimizerScales[0] = DEFAULT_XROT_SCALE;
	optimizerScales[1] = DEFAULT_YROT_SCALE;
	optimizerScales[2] = DEFAULT_ZROT_SCALE;
	optimizerScales[3] = DEFAULT_XTRANS_SCALE;
	optimizerScales[4] = DEFAULT_YTRANS_SCALE;
	optimizerScales[5] = DEFAULT_ZTRANS_SCALE;
	
	transformParams.Fill(0.0);	
}

int PetSpectFusionApp::processCommandLine(int argc, char** argv)
{
	logging::logMessage("Processing command line options");
	
	//command line loop
	for(int i = 1; i < argc; i++)
	{
		if(strcmp(argv[i], "-a") == 0)
		{ 
			automatic_mode = true;
		}		
		else if(strcmp(argv[i], "-b") == 0 && i < argc - 1)
		{
			bins = (int) strtol(argv[++i], NULL, 10);
			
			//check if an error occured
			if(bins == 0)
			{
				std::cout << "Error processing command option: " << argv[i-1] << "!!" << std::endl;
				print_help(argv[0]);
				return 1;
			}
		}
		else if(strcmp(argv[i], "-c") == 0)
		{
			thread_count = (int) strtol(argv[++i], NULL, 10);
			
			//check if an error occured
			if(thread_count == 0)
			{
				std::cout << "Error processing command option: " << argv[i-1] << "!!" << std::endl;
				print_help(argv[0]);
				return 1;
			}
		}
		else if(strcmp(argv[i], "-d") == 0)
		{
			transform_mode = true;
		}
		else if(strcmp(argv[i], "-f") == 0 && i < argc - 1)
		{
			fixedFileName = argv[++i];
		}
		else if(strcmp(argv[i], "-g") == 0)
		{
			graphics_mode = true;
		}
		else if(strcmp(argv[i], "-h") == 0)
		{
			print_help(argv[0]);
			return 1;
		}
		else if(strcmp(argv[i], "-i") == 0 && i < argc - 1)
		{
			maxIterations = (int) strtol(argv[++i], NULL, 10);
			if(maxIterations == 0)
			{
				std::cout << "Error processing command option: " << argv[i-1] << "!!" << std::endl;
				print_help(argv[0]);
				return 1;
			}
		}
		else if(strcmp(argv[i], "-j") == 0 && i < argc - 1)
		{
			minStepSize = strtod(argv[++i], NULL);
			
			if(minStepSize == 0.0)
			{
				std::cout << "Error processing command option: " << argv[i-1] << "!!" << std::endl;
				print_help(argv[0]);
				return 1;
			}
			
		}
		else if(strcmp(argv[i], "-k") == 0 && i < argc - 1)
		{
			maxStepSize = strtod(argv[++i], NULL);
			
			if(maxStepSize == 0.0)
			{
				std::cout << "Error processing command option: " << argv[i-1] << "!!" << std::endl;
				print_help(argv[0]);
				return 1;
			}
		}
		else if(strcmp(argv[i], "-l" ) == 0 && i < argc - 1)
		{
			logFileName = argv[++i];
		}
		else if(strcmp(argv[i], "-m") == 0 && i < argc - 1)
		{
			movingFileName = argv[++i];
		}
		else if(strcmp(argv[i], "-o") == 0 && i < argc - 1)
		{
			outputFileName = argv[++i];
		}
		else if(strcmp(argv[i], "-p") == 0 && i < argc - 1)
		{
			plotFileName = argv[++i];
		}
		else if(strcmp(argv[i], "-r" ) == 0 && i < argc - 1)
		{
			doMultiRes = true;
			levels = (int) strtol(argv[++i], NULL, 10);
			
			if(levels == 0)
			{
				std::cout << "Error processing command option: " << argv[i-1] << "!!" << std::endl;
				print_help(argv[0]);
				return 1;
			}
		}
		else if(strcmp(argv[i], "-s") == 0 && i < argc - 1)
		{
			sampleRate = strtod(argv[++i], NULL);
			
			if(sampleRate == 0.0)
			{
				std::cout << "Error processing command option: " << argv[i-1] << "!!" << std::endl;
				print_help(argv[0]);
				return 1;
			}	
		}
		else if(strcmp(argv[i], "-t") == 0 && i < argc - 1)
		{
			transform = TransformType::New();
			
			for(int j = i+1; j <= i+6 && j < argc; j++)
			{
				transformParams.SetElement(j - i - 1, strtod(argv[j], NULL));
 			}
			
			transform->SetParameters(transformParams);
			
			i += 6;
			
		}	
		else if(strcmp(argv[i], "-tf") == 0 && i < argc - 1)
		{
			TransformType::Pointer transform = TransformType::New();
			std::string fileName = argv[++i];
			readTransformFile(fileName, transform);
			transformParams = transform->GetParameters();
			
		}			
		else if(strcmp(argv[i], "-z") == 0 && i < argc - 1)
		{
			for(int j= i+1; j <= i+6 && j < argc; j++)
			{
				optimizerScales[j - i - 1] = strtod(argv[j], NULL);
				if(optimizerScales[j - i - 1] == 0.0)
				{
					std::cout << "Error processing command option: " << argv[i] << "!!" << std::endl;
					print_help(argv[0]);
					return 1;
				}
			} 
			
			i += 6;
			
		}
		else if(strcmp(argv[i], "-zf") == 0 && i < argc - 1)
		{
			zoomFactor = (int) strtol(argv[++i], NULL, 10);
			
			if((zoomFactor == 0) || (zoomFactor > 6) || (zoomFactor == 5)) {
				std::cout << "Error processing command option: " << argv[i-1] << "!!" << std::endl;
				print_help(argv[0]);
				return 1;
			}
		}		
		//apparently apply sends apps opend with the application launcher a command line paramerter starting with -psn
		//not really sure what is does, couldn't find any infos on the internet about it
		else if(strncmp(argv[i], "-psn", 4) == 0)
		{
			//make sure we still launch into graphics mode
			graphics_mode = true;
		}
		else
		{
			std::cout << "Unrecognized option or invalid usage: " << argv[i] << "!!!" << std::endl;
			std::cout << "Use the option -h to see the list of available command line parameters" << std::endl << std::endl;
			print_help(argv[0]);
			return 1;
		}
	}	

	return 0;
}

void print_help(const char* name)
{
	std::cout << "Command usage modes:" << std::endl;
	std::cout << name << " -f fixed image -m moving image -o output image [-l logfile name] [-b bins] [-c threads]" << std::endl;
	std::cout << " [-s sample rate] [-k min step size] [-l max step size] [-i max iterations] [-p plotfile name]" << std::endl;
	std::cout << " [-r multiresolution levels] [-t trasform parameters] [-tf transform file] [-z optimizer scales]" << std::endl;
	std::cout << " [-zf zoom factor]" << std::endl;
	std::cout << "Starts the programm in fully automated command line only mode" << std::endl << std::endl;
	std::cout << name << " -a -g  -f fixed image -m moving image -o output image [-l logfile name] [-b bins] [-c threads]" << std::endl;
	std::cout << " [-s sample rate] [-j min step size] [-k max step size] [-i max iterations] [-p plotfile name]" << std::endl;
	std::cout << " [-r multiresolution levels] [-t trasform parameters] [-tf transform file] [-z optimizer scales]" << std::endl;
	std::cout << " [-zf zoom factor]" << std::endl;
	std::cout << "Starts the programm in fully automated mode with the graphical interface" << std::endl << std::endl;
	std::cout << name << " -d -f fixed image -m moving image -o output image [-tf transform file || -t transform params]" << std::endl;
	std::cout << "Applies a rigid transform with the given parameters to the moving image, resamples it, and exits" << std::endl << std::endl;
	std::cout << name << " -g -f fixed image -m moving image -o output image [-l logfile name] [-b bins] [-c threads]" << std::endl;
	std::cout << " [-s sample rate] [-k min step size] [-l max step size] [-i max iterations] [-p plotfile name]" << std::endl;
	std::cout << " [-r multiresolution levels] [-t trasform parameters] [-tf transform file] [-z optimizer scales]" << std::endl;
	std::cout << " [-zf zoom factor]" << std::endl;
	std::cout << "Starts the programm with the graphical interface and the given parameters" << std::endl << std::endl;
	std::cout << name << std::endl;
	std::cout << "Starts the graphical interface with the default settings" << std::endl << std::endl;
	std::cout << "Options:" <<std::endl;
	std::cout << "   -a \t Puts the application into automatic mode, i.e. close on completion" << std::endl;
	std::cout << "   -b \t Sets the number of bins used by the registration algorithm [30 - 80]" << std::endl;
	std::cout << "   -c \t Set the global thread count for all multithreaded code [1 - 128]" << std::endl;
	std::cout << "   -d \t Do Transform only. Applies the transform to moving image, saves the result and exits." << std::endl;
	std::cout << "   -f \t Sets the fixed image" << std::endl;
	std::cout << "   -g \t Turn on the graphical user interface" << std::endl;
	std::cout << "   -h \t Displays this message" << std::endl;
	std::cout << "   -i \t Sets the maximum number of iterations used by the registration [250 - 500]" << std::endl;
	std::cout << "   -j \t Sets the minimum step size taken by the optimizer [0.001 - 1.0]" << std::endl;
	std::cout << "   -k \t Sets the maximum step size taken by the optimizer [0.1 - 4.0]" << std::endl;
	std::cout << "   -l \t Turns on logging with the passed logfile name" << std::endl;
	std::cout << "   -m \t Sets the moving image" << std::endl;
	std::cout << "   -o \t Sets the output image name" << std::endl;
	std::cout << "   -p \t Turns on Gnuplot data file generation with the plotfile name" << std::endl;
	std::cout << "   -r \t Turns on multiresolution registration mode, and sets the number of levels [3]" << std::endl;
	std::cout << "   -s \t Sets the sample rate [0.01 - 1.0]" << std::endl;
	std::cout << "   -t \t Sets the initial transform parameters used by the registration, example:" << std::endl;
	std::cout << "\t\t -t xRotation yRotation zRotation xTranslation yTranslation zTranslation" << std::endl;
	std::cout << "   -tf \t Sets the initial transform parameters using the transform file generated from a previous registration" << std::endl;
	std::cout << "   -z \t Sets the optimizer scales used by the registration, example:" << std::endl;
	std::cout << "\t\t -z xRotation yRotation zRotatation xTranslation yTranslation zTranslation" << std::endl;
	std::cout << "\t\t Translation range [0.001 - 0.000001] Rotation range [1.0 - 10.0]" << std::endl; 
	std::cout << "   -zf \t Sets the initial zoom factor of the image display [1 - 4, 6]" << std::endl;
	std::cout << "PetSpectFusion v1.30 (c) 2008, 2009 Brian Jensen, Nuklearmedizinische Klink der TU Muenchen" << std::endl;
	std::cout << "For more information contact Brian Jensen, jensen@in.tum.de" << std::endl;
	
}
